package com.nantian.cloud.dbcapp.carbon.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nantian.cloud.dbcapp.carbon.domain.ent.InstrumentInfo;
import io.swagger.models.auth.In;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 计量器具信息 Mapper 接口
 * </p>
 *
 * @author wujing
 * @since 2022-09-28
 */
@Mapper
public interface InstrumentInfoDao extends BaseMapper<InstrumentInfo> {

    List<InstrumentInfo> selectByEntId(@Param("entCode") String entCode);

    InstrumentInfo selectById(@Param("entCode") String entCode, @Param("id") Integer id);

    IPage<InstrumentInfo> getList(Page page,@Param("instrumentInfo") InstrumentInfo instrumentInfo);

    List<InstrumentInfo> listAll(@Param("instrumentInfo") InstrumentInfo instrumentInfo);

    int saveInfo(@Param("instrumentInfo") InstrumentInfo instrumentInfo);

    int updateInfoById(@Param("instrumentInfo") InstrumentInfo instrumentInfo);


}
